@@ -33,10 +33,10 @@
  * w) representing rows, and elements of each vector are a column:
  *
  * |[<!-- language="plain" -->
- *   | x |    | x.x   x.y   x.z   x.w |
- *   | y | -\ | y.x   y.y   y.z   y.w |
- *   | z | -/ | z.x   z.y   z.z   z.w |
- *   | w |    | w.x   w.y   w.z   w.w |
+ *   ⎡ m.x ⎤    ⎛ x.x  x.y  x.z  x.w ⎞
+ *   ⎜ m.y ⎟ -\ ⎜ y.x  y.y  y.z  y.w ⎟
+ *   ⎜ m.z ⎟ -/ ⎜ z.x  z.y  z.z  z.w ⎟
+ *   ⎣ m.w ⎦    ⎝ w.x  w.y  w.z  w.w ⎠
  * ]|
  *
  * It is possible to easily convert a #graphene_matrix_t to and from an array
@@ -386,8 +386,8 @@ graphene_matrix_init_translate (graphene_matrix_t        *m,
 /**
  * graphene_matrix_init_skew:
  * @m: a #graphene_matrix_t
- * @x_skew: skew factor on the X axis
- * @y_skew: skew factor on the Y axis
+ * @x_skew: skew factor, in radians, on the X axis
+ * @y_skew: skew factor, in radians, on the Y axis
  *
  * Initializes a #graphene_matrix_t with a skew transformation
  * with the given factors.
@@ -560,13 +560,13 @@ graphene_matrix_is_singular (const graphene_matrix_t *m)
  * The arguments map to the following matrix layout:
  *
  * |[<!-- language="plain" -->
- *   | xx yx |   |  a  b  0 |
- *   | xy yy | = |  c  d  0 |
- *   | x0 y0 |   | tx ty  1 |
+ *   ⎛ xx  yx ⎞   ⎛  a   b  0 ⎞
+ *   ⎜ xy  yy ⎟ = ⎜  c   d  0 ⎟
+ *   ⎝ x0  y0 ⎠   ⎝ tx  ty  1 ⎠
  * ]|
  *
- * This function can be used to convert between a matrix type from
- * other libraries and a #graphene_matrix_t.
+ * This function can be used to convert between an affine matrix type
+ * from other libraries and a #graphene_matrix_t.
  *
  * Returns: (transfer none): the initialized matrix
  *
@@ -605,13 +605,13 @@ graphene_matrix_init_from_2d (graphene_matrix_t *m,
  * The returned values have the following layout:
  *
  * |[<!-- language="plain" -->
- *   | xx yx |   |  a  b  0 |
- *   | xy yy | = |  c  d  0 |
- *   | x0 y0 |   | tx ty  1 |
+ *   ⎛ xx  yx ⎞   ⎛  a   b  0 ⎞
+ *   ⎜ xy  yy ⎟ = ⎜  c   d  0 ⎟
+ *   ⎝ x0  y0 ⎠   ⎝ tx  ty  1 ⎠
  * ]|
  *
  * This function can be used to convert between a #graphene_matrix_t
- * and a matrix type from other libraries.
+ * and an affine matrix type from other libraries.
  *
  * Returns: `true` if the matrix is compatible with an affine
  *   transformation matrix
@@ -629,38 +629,22 @@ graphene_matrix_to_2d (const graphene_matrix_t *m,
 {
   float res[4];
 
-  graphene_simd4f_dup_4f (m->value.x, res);
-  if (!graphene_approx_val (res[2], 0.f) &&
-      !graphene_approx_val (res[3], 0.f))
+  if (!graphene_simd4x4f_is_2d (&m->value))
     return false;
 
+  graphene_simd4f_dup_4f (m->value.x, res);
   if (xx != NULL)
     *xx = res[0];
   if (yx != NULL)
     *yx = res[1];
 
   graphene_simd4f_dup_4f (m->value.y, res);
-  if (!graphene_approx_val (res[2], 0.f) &&
-      !graphene_approx_val (res[3], 0.f))
-    return false;
-
   if (xy != NULL)
     *xy = res[0];
   if (yy != NULL)
     *yy = res[1];
 
-  graphene_simd4f_dup_4f (m->value.z, res);
-  if (!graphene_approx_val (res[0], 0.f) &&
-      !graphene_approx_val (res[1], 0.f) &&
-      !graphene_approx_val (res[2], 1.f) &&
-      !graphene_approx_val (res[3], 0.f))
-    return false;
-
   graphene_simd4f_dup_4f (m->value.w, res);
-  if (!graphene_approx_val (res[2], 0.f) &&
-      !graphene_approx_val (res[3], 1.f))
-    return false;
-
   if (x_0 != NULL)
     *x_0 = res[0];
   if (y_0 != NULL)
@@ -783,8 +767,8 @@ graphene_matrix_get_value (const graphene_matrix_t *m,
  *
  * Multiplies two #graphene_matrix_t.
  *
- * Remember: matrix multiplication is not commutative, except for the
- * identity matrix; the product of this multiplication is (A * B).
+ * Matrix multiplication is not commutative in general; the order of the factors matters.
+ * The product of this multiplication is (@a × @b)
  *
  * Since: 1.0
  */
@@ -824,6 +808,12 @@ graphene_matrix_determinant (const graphene_matrix_t *m)
  *
  * Transforms the given #graphene_vec3_t using the matrix @m.
  *
+ * This function will multiply the X, Y, and Z row vectors of the matrix @m
+ * with the corresponding components of the vector @v. The W row vector will
+ * be ignored.
+ *
+ * See also: graphene_simd4x4f_vec3_mul()
+ *
  * Since: 1.0
  */
 void
@@ -842,6 +832,8 @@ graphene_matrix_transform_vec3 (const graphene_matrix_t *m,
  *
  * Transforms the given #graphene_vec4_t using the matrix @m.
  *
+ * See also: graphene_simd4x4f_vec4_mul()
+ *
  * Since: 1.0
  */
 void
@@ -861,6 +853,12 @@ graphene_matrix_transform_vec4 (const graphene_matrix_t *m,
  *
  * Transforms the given #graphene_point_t using the matrix @m.
  *
+ * Unlike graphene_matrix_transform_vec3(), this function will take into
+ * account the fourth row vector of the #graphene_matrix_t when computing
+ * the dot product of each row vector of the matrix.
+ *
+ * See also: graphene_simd4x4f_point3_mul()
+ *
  * Since: 1.0
  */
 void
@@ -870,8 +868,8 @@ graphene_matrix_transform_point (const graphene_matrix_t *m,
 {
   graphene_simd4f_t vec3;
 
-  vec3 = graphene_simd4f_init (p->x, p->y, 0.0f, 0.0f);
-  graphene_simd4x4f_vec3_mul (&m->value, &vec3, &vec3);
+  vec3 = graphene_simd4f_init (p->x, p->y, 0.0f, 1.0f);
+  graphene_simd4x4f_point3_mul (&m->value, &vec3, &vec3);
 
   res->x = graphene_simd4f_get_x (vec3);
   res->y = graphene_simd4f_get_y (vec3);
@@ -886,7 +884,8 @@ graphene_matrix_transform_point (const graphene_matrix_t *m,
  * Transforms the given #graphene_point3d_t using the matrix @m.
  *
  * Unlike graphene_matrix_transform_vec3(), this function will take into
- * account the fourth row vector of the #graphene_matrix_t.
+ * account the fourth row vector of the #graphene_matrix_t when computing
+ * the dot product of each row vector of the matrix.
  *
  * Since: 1.2
  */
@@ -897,7 +896,7 @@ graphene_matrix_transform_point3d (const graphene_matrix_t  *m,
 {
   graphene_simd4f_t vec3;
 
-  vec3 = graphene_simd4f_init (p->x, p->y, p->z, 0.f);
+  vec3 = graphene_simd4f_init (p->x, p->y, p->z, 1.f);
   graphene_simd4x4f_point3_mul (&m->value, &vec3, &vec3);
 
   res->x = graphene_simd4f_get_x (vec3);
@@ -912,8 +911,9 @@ graphene_matrix_transform_point3d (const graphene_matrix_t  *m,
  * @res: (out caller-allocates): return location for the
  *   transformed quad
  *
- * Transforms a #graphene_rect_t using the given matrix @m. The
- * result is a coplanar quad.
+ * Transforms each corner of a #graphene_rect_t using the given matrix @m.
+ *
+ * The result is a coplanar quadrilateral.
  *
  * Since: 1.0
  */
@@ -950,8 +950,10 @@ graphene_matrix_transform_rect (const graphene_matrix_t *m,
  * @res: (out caller-allocates): return location for the bounds
  *   of the transformed rectangle
  *
- * Transforms a #graphene_rect_t using the given matrix @m. The
- * result is the bounding box containing the coplanar quad.
+ * Transforms each corner of a #graphene_rect_t using the given matrix @m.
+ *
+ * The result is the axis aligned bounding rectangle containing the coplanar
+ * quadrilateral.
  *
  * Since: 1.0
  */
@@ -1047,8 +1049,10 @@ graphene_matrix_transform_sphere (const graphene_matrix_t *m,
  * @res: (out caller-allocates): return location for the bounds
  *   of the transformed box
  *
- * Transforms a #graphene_box_t using the given matrix @m. The
- * result is the bounding box containing the transformed box.
+ * Transforms the vertices of a #graphene_box_t using the given matrix @m.
+ *
+ * The result is the axis aligned bounding box containing the transformed
+ * vertices.
  *
  * Since: 1.2
  */
@@ -1138,8 +1142,8 @@ graphene_matrix_project_point (const graphene_matrix_t *m,
  *
  * Projects a #graphene_rect_t using the given matrix.
  *
- * The resulting rectangle is the bounding box capable of containing
- * fully the projected rectangle.
+ * The resulting rectangle is the axis aligned bounding rectangle capable
+ * of containing fully the projected rectangle.
  *
  * Since: 1.0
  */
@@ -1214,7 +1218,7 @@ graphene_matrix_project_rect (const graphene_matrix_t *m,
  *   untransformed point
  *
  * Undoes the transformation of a #graphene_point_t using the
- * given matrix, within the given rectangular @bounds.
+ * given matrix, within the given axis aligned rectangular @bounds.
  *
  * Returns: `true` if the point was successfully untransformed
  *
@@ -1258,8 +1262,8 @@ graphene_matrix_untransform_point (const graphene_matrix_t *m,
  * @res: (out caller-allocates): return location for the
  *   untransformed rectangle
  *
- * Undoes the transformation on the points of a #graphene_rect_t
- * using the given matrix, within the given rectangular @bounds.
+ * Undoes the transformation on the corners of a #graphene_rect_t using the
+ * given matrix, within the given axis aligned rectangular @bounds.
  *
  * Since: 1.0
  */
@@ -1343,6 +1347,9 @@ graphene_matrix_unproject_point3d (const graphene_matrix_t  *projection,
  * Adds a translation transformation to @m using the coordinates
  * of the given #graphene_point3d_t.
  *
+ * This is the equivalent of calling graphene_matrix_init_translate() and
+ * then multiplying @m with the translation matrix.
+ *
  * Since: 1.0
  */
 void
@@ -1363,6 +1370,9 @@ graphene_matrix_translate (graphene_matrix_t        *m,
  * Adds a rotation transformation to @m, using the given
  * #graphene_quaternion_t.
  *
+ * This is the equivalent of calling graphene_quaternion_to_matrix() and
+ * then multiplying @m with the rotation matrix.
+ *
  * Since: 1.2
  */
 void
@@ -1415,6 +1425,9 @@ graphene_matrix_rotate_internal (graphene_simd4x4f_t     *m,
  * Adds a rotation transformation to @m, using the given @angle
  * and @axis vector.
  *
+ * This is the equivalent of calling graphene_matrix_init_rotate() and
+ * then multiplying the matrix @m with the rotation matrix.
+ *
  * Since: 1.0
  */
 void
@@ -1433,6 +1446,8 @@ graphene_matrix_rotate (graphene_matrix_t     *m,
  * Adds a rotation transformation around the X axis to @m, using
  * the given @angle.
  *
+ * See also: graphene_matrix_rotate()
+ *
  * Since: 1.0
  */
 void
@@ -1451,6 +1466,8 @@ graphene_matrix_rotate_x (graphene_matrix_t *m,
  * Adds a rotation transformation around the Y axis to @m, using
  * the given @angle.
  *
+ * See also: graphene_matrix_rotate()
+ *
  * Since: 1.0
  */
 void
@@ -1469,6 +1486,8 @@ graphene_matrix_rotate_y (graphene_matrix_t *m,
  * Adds a rotation transformation around the Z axis to @m, using
  * the given @angle.
  *
+ * See also: graphene_matrix_rotate()
+ *
  * Since: 1.0
  */
 void
@@ -1489,6 +1508,9 @@ graphene_matrix_rotate_z (graphene_matrix_t *m,
  * Adds a scaling transformation to @m, using the three
  * given factors.
  *
+ * This is the equivalent of calling graphene_matrix_init_scale() and then
+ * multiplying the matrix @m with the scale matrix.
+ *
  * Since: 1.0
  */
 void
@@ -1666,6 +1688,54 @@ graphene_matrix_normalize (const graphene_matrix_t *m,
   res->value.w = graphene_simd4f_mul (m->value.w, n);
 }
 
+/**
+ * graphene_matrix_get_x_translation:
+ * @m: a #graphene_matrix_t
+ *
+ * Retrieves the translation component on the X axis from @m.
+ *
+ * Returns: the translation component
+ *
+ * Since: 1.10
+ */
+float
+graphene_matrix_get_x_translation (const graphene_matrix_t *m)
+{
+  return graphene_simd4f_get_x (m->value.w);
+}
+
+/**
+ * graphene_matrix_get_y_translation:
+ * @m: a #graphene_matrix_t
+ *
+ * Retrieves the translation component on the Y axis from @m.
+ *
+ * Returns: the translation component
+ *
+ * Since: 1.10
+ */
+float
+graphene_matrix_get_y_translation (const graphene_matrix_t *m)
+{
+  return graphene_simd4f_get_y (m->value.w);
+}
+
+/**
+ * graphene_matrix_get_z_translation:
+ * @m: a #graphene_matrix_t
+ *
+ * Retrieves the translation component on the Z axis from @m.
+ *
+ * Returns: the translation component
+ *
+ * Since: 1.10
+ */
+float
+graphene_matrix_get_z_translation (const graphene_matrix_t *m)
+{
+  return graphene_simd4f_get_z (m->value.w);
+}
+
 /**
  * graphene_matrix_get_x_scale:
  * @m: a #graphene_matrix_t
@@ -1822,7 +1892,7 @@ matrix_decompose_3d (const graphene_matrix_t *m,
                      graphene_point3d_t      *translate_r,
                      graphene_vec4_t         *perspective_r)
 {
-  graphene_matrix_t local, perspective;
+  graphene_matrix_t local;
   float shear_xy, shear_xz, shear_yz;
   float scale_x, scale_y, scale_z;
   graphene_simd4f_t perspective_v;
@@ -1840,17 +1910,20 @@ matrix_decompose_3d (const graphene_matrix_t *m,
    * but it also provides an easy way to test for singularity of
    * the upper 3x3 component
    */
-  perspective = local;
-  perspective.value.w = graphene_simd4f_init (0.f, 0.f, 0.f, 1.f);
-
-  if (graphene_approx_val (graphene_matrix_determinant (&perspective), 0.f))
-    return false;
-
   perspective_v = graphene_simd4f_init (graphene_simd4f_get_w (local.value.x),
                                         graphene_simd4f_get_w (local.value.y),
                                         graphene_simd4f_get_w (local.value.z),
                                         graphene_simd4f_get_w (local.value.w));
 
+  /* Clear the perspective component */
+  local.value.x = graphene_simd4f_merge_w (local.value.x, 0.f);
+  local.value.y = graphene_simd4f_merge_w (local.value.y, 0.f);
+  local.value.z = graphene_simd4f_merge_w (local.value.z, 0.f);
+  local.value.w = graphene_simd4f_merge_w (local.value.w, 1.f);
+
+  if (graphene_approx_val (graphene_matrix_determinant (&local), 0.f))
+    return false;
+
   /* isolate the perspective */
   if (!graphene_simd4f_is_zero3 (perspective_v))
     {
@@ -1864,14 +1937,8 @@ matrix_decompose_3d (const graphene_matrix_t *m,
        * check if the matrix is invertible here because we just checked
        * whether the determinant is not zero.
        */
-      graphene_matrix_inverse (&perspective, &tmp);
-      graphene_matrix_transpose_transform_vec4 (&tmp, perspective_r, perspective_r);
-
-      /* Clear the perspective component */
-      local.value.x = graphene_simd4f_merge_w (local.value.x, 0.f);
-      local.value.y = graphene_simd4f_merge_w (local.value.y, 0.f);
-      local.value.z = graphene_simd4f_merge_w (local.value.z, 0.f);
-      local.value.w = graphene_simd4f_merge_w (local.value.w, 1.f);
+      graphene_matrix_inverse (&local, &tmp);
+      graphene_matrix_transform_vec4 (&tmp, perspective_r, perspective_r);
     }
   else
     graphene_vec4_init (perspective_r, 0.f, 0.f, 0.f, 1.f);
@@ -2139,3 +2206,101 @@ graphene_matrix_print (const graphene_matrix_t *m)
                graphene_matrix_get_value (m, i, 3));
     }
 }
+
+/**
+ * graphene_matrix_near:
+ * @a: a #graphene_matrix_t
+ * @b: a #graphene_matrix_t
+ * @epsilon: the threshold between the two matrices 
+ *
+ * Compares the two given #graphene_matrix_t matrices and checks
+ * whether their values are within the given @epsilon of each
+ * other.
+ *
+ * Returns: `true` if the two matrices are near each other, and
+ *   `false` otherwise
+ *
+ * Since: 1.10
+ */
+bool
+graphene_matrix_near (const graphene_matrix_t *a,
+                      const graphene_matrix_t *b,
+                      float                    epsilon)
+{
+  if (a == b)
+    return true;
+
+  if (a == NULL || b == NULL)
+    return false;
+
+  for (unsigned i = 0; i < 4; i++)
+    {
+      graphene_vec4_t row_a, row_b;
+
+      graphene_matrix_get_row (a, i, &row_a);
+      graphene_matrix_get_row (b, i, &row_b);
+
+      if (!graphene_vec4_near (&row_a, &row_b, epsilon))
+        return false;
+    }
+
+  return true;
+}
+
+/**
+ * graphene_matrix_equal:
+ * @a: a #graphene_matrix_t
+ * @b: a #graphene_matrix_t
+ *
+ * Checks whether the two given #graphene_matrix_t matrices are equal.
+ *
+ * Returns: `true` if the two matrices are equal, and `false` otherwise
+ *
+ * Since: 1.10
+ */
+bool
+graphene_matrix_equal (const graphene_matrix_t *a,
+                       const graphene_matrix_t *b)
+{
+  return graphene_matrix_near (a, b, FLT_EPSILON);
+}
+
+/**
+ * graphene_matrix_equal_fast:
+ * @a: a #graphene_matrix_t
+ * @b: a #graphene_matrix_t
+ *
+ * Checks whether the two given #graphene_matrix_t matrices are
+ * byte-by-byte equal.
+ *
+ * While this function is faster than graphene_matrix_equal(), it
+ * can also return false negatives, so it should be used in
+ * conjuction with either graphene_matrix_equal() or
+ * graphene_matrix_near(). For instance:
+ *
+ * |[<!-- language="C" -->
+ *   if (graphene_matrix_equal_fast (a, b))
+ *     {
+ *       // matrices are definitely the same
+ *     }
+ *   else
+ *     {
+ *       if (graphene_matrix_equal (a, b))
+ *         // matrices contain the same values within an epsilon of FLT_EPSILON
+ *       else if (graphene_matrix_near (a, b, 0.0001))
+ *         // matrices contain the same values within an epsilon of 0.0001
+ *       else
+ *         // matrices are not equal
+ *     }
+ * ]|
+ *
+ * Returns: `true` if the matrices are equal. and `false` otherwise
+ *
+ * Since: 1.10
+ */
+bool
+graphene_matrix_equal_fast (const graphene_matrix_t *a,
+                            const graphene_matrix_t *b)
+{
+  return memcmp (a, b, sizeof (graphene_matrix_t)) == 0;
+}
