@@ -267,6 +267,13 @@ GRAPHENE_AVAILABLE_IN_1_0
 void                    graphene_matrix_normalize               (const graphene_matrix_t  *m,
                                                                  graphene_matrix_t        *res);
 
+GRAPHENE_AVAILABLE_IN_1_10
+float                   graphene_matrix_get_x_translation       (const graphene_matrix_t  *m);
+GRAPHENE_AVAILABLE_IN_1_10
+float                   graphene_matrix_get_y_translation       (const graphene_matrix_t  *m);
+GRAPHENE_AVAILABLE_IN_1_10
+float                   graphene_matrix_get_z_translation       (const graphene_matrix_t  *m);
+
 GRAPHENE_AVAILABLE_IN_1_0
 float                   graphene_matrix_get_x_scale             (const graphene_matrix_t  *m);
 GRAPHENE_AVAILABLE_IN_1_0
@@ -280,6 +287,17 @@ void                    graphene_matrix_interpolate             (const graphene_
                                                                  double                    factor,
                                                                  graphene_matrix_t        *res);
 
+GRAPHENE_AVAILABLE_IN_1_10
+bool                    graphene_matrix_near                    (const graphene_matrix_t  *a,
+                                                                 const graphene_matrix_t  *b,
+                                                                 float                     epsilon);
+GRAPHENE_AVAILABLE_IN_1_10
+bool                    graphene_matrix_equal                   (const graphene_matrix_t  *a,
+                                                                 const graphene_matrix_t  *b);
+GRAPHENE_AVAILABLE_IN_1_10
+bool                    graphene_matrix_equal_fast              (const graphene_matrix_t  *a,
+                                                                 const graphene_matrix_t  *b);
+
 GRAPHENE_AVAILABLE_IN_1_0
 void                    graphene_matrix_print                   (const graphene_matrix_t  *m);
 
