@@ -156,6 +156,22 @@ graphene_rect_init_from_rect (graphene_rect_t       *r,
   return r;
 }
 
+static bool
+rect_equal (const void *p1,
+            const void *p2)
+{
+  const graphene_rect_t *a = p1;
+  const graphene_rect_t *b = p2;
+
+  graphene_rect_t r_a, r_b;
+
+  graphene_rect_normalize_r (a, &r_a);
+  graphene_rect_normalize_r (b, &r_b);
+
+  return graphene_point_equal (&r_a.origin, &r_b.origin) &&
+         graphene_size_equal (&r_a.size, &r_b.size);
+}
+
 /**
  * graphene_rect_equal:
  * @a: a #graphene_rect_t
@@ -171,20 +187,7 @@ bool
 graphene_rect_equal (const graphene_rect_t *a,
                      const graphene_rect_t *b)
 {
-  if (a == b)
-    return true;
-
-  if (a == NULL || b == NULL)
-    return false;
-
-  graphene_rect_t r_a = *a;
-  graphene_rect_t r_b = *b;
-
-  graphene_rect_normalize_in_place (&r_a);
-  graphene_rect_normalize_in_place (&r_b);
-
-  return graphene_point_equal (&r_a.origin, &r_b.origin) &&
-         graphene_size_equal (&r_a.size, &r_b.size);
+  return graphene_pointer_equal (a, b, rect_equal);
 }
 
 /**
@@ -243,8 +246,8 @@ graphene_rect_normalize_r (const graphene_rect_t *r,
  * Since: 1.0
  */
 void
-graphene_rect_get_center (const graphene_rect_t  *r,
-                          graphene_point_t       *p)
+graphene_rect_get_center (const graphene_rect_t *r,
+                          graphene_point_t      *p)
 {
   graphene_rect_t rr;
 
@@ -329,8 +332,8 @@ graphene_rect_get_bottom_left (const graphene_rect_t *r,
  * Since: 1.0
  */
 void
-graphene_rect_get_bottom_right (const graphene_rect_t  *r,
-                                graphene_point_t       *p)
+graphene_rect_get_bottom_right (const graphene_rect_t *r,
+                                graphene_point_t      *p)
 {
   graphene_rect_t rr;
 
@@ -366,17 +369,17 @@ graphene_rect_get_vertices (const graphene_rect_t *r,
   graphene_vec2_init (&vertices[3], rr.origin.x, rr.origin.y + rr.size.height);
 }
 
-#define GRAPHENE_RECT_GET(arg,part,field) \
-float \
-graphene_rect_get_##field (const graphene_rect_t *arg) \
-{ \
-  graphene_rect_t rr; \
+#define GRAPHENE_RECT_GET(arg, part, field) \
+  float \
+  graphene_rect_get_ ## field (const graphene_rect_t * arg) \
+  { \
+    graphene_rect_t rr; \
 \
-  rr = *arg; \
-  graphene_rect_normalize_in_place (&rr); \
+    rr = *arg; \
+    graphene_rect_normalize_in_place (&rr); \
 \
-  return rr.part.field; \
-}
+    return rr.part.field; \
+  }
 
 /**
  * graphene_rect_get_x:
@@ -826,3 +829,32 @@ graphene_rect_zero (void)
 {
   return &_graphene_rect_zero;
 }
+
+/**
+ * graphene_rect_scale:
+ * @r: a #graphene_rect_t
+ * @s_h: horizontal scale factor
+ * @s_v: vertical scale factor
+ * @res: (out caller-allocates): return location for the
+ *   scaled rectangle
+ *
+ * Scales the size and origin of a rectangle horizontaly by @s_h,
+ * and vertically by @s_v. The result @res is normalized.
+ *
+ * Since: 1.10
+ */
+void
+graphene_rect_scale (const graphene_rect_t *r,
+                     float                  s_h,
+                     float                  s_v,
+                     graphene_rect_t       *res)
+{
+  graphene_rect_normalize_r (r, res);
+
+  res->origin.x *= s_h;
+  res->origin.y *= s_v;
+  res->size.width *= s_h;
+  res->size.height *= s_v;
+
+  graphene_rect_normalize_r (res, res);
+}
